/*
 *  $Id$
 *  Copyright (c) 2003, Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>
 *
 *  MoC - Master of Celebration is an automatic playlist generation tool using
 *  state-of-the-art music similarity measures
 *  http://www.schnitzer.at/dominik/moc/
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <unistd.h>
#include <gst/gst.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "audiofilein.h"
#include "gstreplaygain.h"

using namespace std;

audiofilein::audiofilein()
{
    _gst_info_categories = 0;
    gst_init(0, 0);
}

audiofilein::~audiofilein()
{
}

string
audiofilein::decode(const string &filename, int srate, double *rpgainscale)
{
    // check if file exists
    struct stat buf;
    double rpgain;
    
    if (stat(filename.c_str(), &buf) == -1)
        return "";

    GstElement *pipeline, *filesrc, *decoder, *s2m, *audioscale, *fdsink,
        *replaygain, *i2f;
    char tmpf[] = "/tmp/moc-XXXXXX";
    int fd = mkstemp(tmpf);

    // create a new pipeline/filesink
    pipeline = gst_pipeline_new("pipeline");
    filesrc = gst_element_factory_make("filesrc", "disk_source");
    g_object_set(G_OBJECT(filesrc), "location", filename.c_str(), NULL);

    // get the decoder/audio sink
    decoder = gst_element_factory_make("mad", "decoder");
    s2m = gst_element_factory_make("stereo2mono", "converter1");
    audioscale = gst_element_factory_make("audioscale", "converter2");
    g_object_set(G_OBJECT(audioscale), "frequency", srate, NULL);
    i2f = gst_element_factory_make("int2float", "converter3");
    replaygain = gst_element_factory_make("replaygain", "analyzer1");
    g_object_set(G_OBJECT(replaygain), "silent", true, NULL);

    fdsink = gst_element_factory_make("fdsink", "write_audio");
    g_object_set(G_OBJECT(fdsink), "fd", fd, NULL);

    // add objects to the main pipeline
    gst_bin_add_many(GST_BIN (pipeline), filesrc, decoder, s2m,
            audioscale, replaygain, i2f, fdsink, NULL);
    gst_element_link_many(filesrc, decoder, s2m, audioscale,
            replaygain, i2f, fdsink, NULL);

    // start playing
    gst_element_set_state (pipeline, GST_STATE_PLAYING);
    while (gst_bin_iterate(GST_BIN (pipeline)));
        
    // stop the pipeline
    gst_element_set_state(pipeline, GST_STATE_NULL);
    g_object_get (G_OBJECT(replaygain), "rpgain", &rpgain, NULL);
    *rpgainscale = pow(10.0, rpgain/20.0);

    // unreffing the pipeline unrefs the contained elements as well
    gst_object_unref(GST_OBJECT(pipeline));

    close(fd);

    return string(tmpf);
}

