/*
 *  $Id$
 *  Copyright (c) 2003, Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>
 *
 *  MoC - Master of Celebration is an automatic playlist generation tool using
 *  state-of-the-art music similarity measures
 *  http://www.schnitzer.at/dominik/moc/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __GMM_H
#define __GMM_H

#include <valarray>
#include <iostream>
#include <string>
#include <map>
#include <utility>
#include <vector>
#include "vec.h"
#include "mat.h"

using namespace std;

class gmm
{
    private:
        int datacount;
        
        string filename;
        string md5sum;
        int gaussians;
        int gaussiandim;
        vec priors;
        mat means;
        mat covs;
        double selfdist;

        map<gmm*, double> distcache;

        void sample_data();

    public:
        gmm(const string &fname, const string &md5, vec &prior,
                mat &mean, mat &cov);
        gmm(const string &fname);
        ~gmm();

        double distance(gmm *g, int datacount);
        void set_distance(gmm *g, double dist);
        double get_distance(gmm *g, int datacount);
        double get_selfdistance(int datacount);

        int get_gaussians();
        int get_gaussiandim();
        mat* get_means();
        mat* get_covs();
        vec* get_priors();
        string get_filename();
        string get_md5sum();
        void print_model(ostream &c);
        void load_model(const string &fname);
        void save_model(const string &fname);
};

#endif /* __GMM_H */
