/*
 *  $Id$
 *  Copyright (c) 2003, Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>
 *
 *  MoC - Master of Celebration is an automatic playlist generation tool using
 *  state-of-the-art music similarity measures
 *  http://www.schnitzer.at/dominik/moc/
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <string>
#include <limits.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "moc.h"
#include "gmm.h"
#include "mathtools.h"

using namespace std;

extern string dbloc;

void
usage(string prog)
{
    cerr << "Usage: " << prog << " [OPTION]... [FILE]..." << endl;
    cerr << "Master of Celebration - an artificial intelligence playlist" <<
        "generator." << endl;
    cerr << "Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>, $Date$" << endl;
    cerr << endl;
    cerr << "Options:" << endl;
    cerr << "-a\t\tadd files to your moc archive" << endl;
    cerr << "-q SIZE\t\tgenerate a SIZE items playlist using FILE as startpoint"
        << endl;
    cerr << "-v\t\tbe verbose in what moc is doing" << endl;
    cerr << endl;
}

bool
file_readable(string fname)
{
    int fd;
    if ((fd = open(fname.c_str(), O_RDONLY)) != -1) 
    {
        close(fd);
        return true;
    }
    else
        return false;
}

int
main(int argc, char *argv[])
{
    int         c;
    int         opt_dist = 0;
    int         opt_add = 0;
    int         opt_err = 0;
    int         opt_query = 0;
    int         opt_loc = 0;
    int         opt_verb = 0;
    int         querynum = 0;

    string      infile;
    string      distf;
    // Check options
    opterr = 0;
    while ((c = getopt(argc, argv, "d:q:al:v")) != EOF)
    {
        switch (c)
        {
            case 'd':
                if (opt_dist)
                    opt_err = 1;
                else
                {
                    distf = optarg;
                    opt_dist = 1;
                }
                break;
                
            case 'a':
                if (opt_add)
                    opt_err = 1;
                else
                    opt_add = 1;
                break;

            case 'q':
                if (opt_query)
                    opt_err = 1;
                else
                {
                    opt_query = 1;
                    querynum = atol(optarg);
                }
                break;

            case 'l':
                if (opt_loc)
                    opt_err = 1;
                else
                {
                    opt_loc = 1;
                    dbloc = optarg;
                }
                break;

            case 'v':
                if (opt_verb)
                    opt_err = 1;
                else
                    opt_verb = 1;
                break;
                
            case '?':
                opt_err = 1;
                break;

            default:
                opt_err = 1;
                break;
        }
    }

    if ((opt_err == 1) || (optind >= argc) || (opt_query && opt_add))
    {
        usage(argv[0]);
        return EXIT_FAILURE;
    }

    moc m(3, 2048, 22050, 8);

    if (opt_add)
    {
        time_t before, start = time(NULL);
        string fname;
        typedef string::size_type ST;
        ST st;

        cout << "Adding file(s) to library:" << endl;
        while (optind < argc)
        {
            before = time(NULL);
            infile = argv[optind++];

            if (file_readable(infile))
            {
                st = infile.rfind("/");
                if (st == string::npos)
                    fname = infile;
                else
                    fname = infile.substr(st+1, infile.length() - st);
                
                cout << fname << ":" << flush;
                gmm *g = m.get_gmm(infile, 200, 800);
                if (g == NULL)
                {
                    cout << " failied (i.e. mono input)" << endl;
                }
                else
                {
                    m.add_gmm(g);
                    cout << " done (" << (time(NULL) - before)/60 <<
                        ":" << (time(NULL) - before) % 60 << 
                        "/" << (time(NULL) - start)/60 << ":" <<
                        (time(NULL) - start) % 60 << ")" << endl;
                }
            }
            else
                cerr << argv[0] << ": " << infile << " not accessible!" << endl;
        }
    }
    else if(opt_query)
    {
        rnd_init();
        infile = argv[optind];
        if (!file_readable(infile))
        {
            cerr << argv[0] << ": " << infile << " not accessible!" << endl;
            return EXIT_FAILURE;
        }
        gmm *g = m.get_gmm_md5(m.get_md5(infile));
        if (g == NULL)
        {
            cerr << argv[0] << ": " << infile << " not in library, add it to "
                << "query it." << endl;
            return EXIT_FAILURE;
        }


        vector<pair<gmm*, double> > *pl = m.get_nclosest(g, querynum, 2000);
        for (int i = 0; (i < (int)pl->size()); i++)
        {
            cout << ((*pl)[i].first)->get_filename();
            if (opt_verb)
                cout << " (Distance: " << (*pl)[i].second << ")";

            cout << endl;
        }
    }
    else if (opt_dist)
    {
        gmm *g1 = m.get_gmm_md5(m.get_md5(distf));
        gmm *g2 = m.get_gmm_md5(m.get_md5(argv[optind]));

        double dist = g1->get_distance(g2, 2000);
        cout << "File 1: " << distf << endl << "File 2: " <<
            argv[optind] << endl;
        cout << "Distance:" << dist << endl;
    }

    return EXIT_SUCCESS;
}
