/*
 *  $Id$
 *  Copyright (c) 2003, Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>
 *
 *  MoC - Master of Celebration is an automatic playlist generation tool using
 *  state-of-the-art music similarity measures
 *  http://www.schnitzer.at/dominik/moc/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

using namespace std;

#include <cmath>
#include <cstdlib>
#include <valarray>
#include "vec.h"
#include "mathtools.h"

double nrndcache[NRNDCACHE_MAX];

void
rnd_init()
{
    srand(time(NULL));

    for (int i = 0; i < NRNDCACHE_MAX; i++)
        nrndcache[i] = nrnd2(0.0, 1.0);
}
    

double
rnd(double min, double max)
{
    return (double)(rand()*(max - min))/RAND_MAX + min;
}

double
nrnd(const double& mean, const double& sdev)
{
    double gset, fac, r, v1, v2;

    do
    {
        v1 = rnd(-1.0, 1.0);
        v2 = rnd(-1.0, 1.0);
        r = v1*v1 + v2*v2;
    }
    while (r >= 1.0);
    fac = sqrt(-2.0*log(r)/r);
    gset = v1*fac;

    return mean + v2*fac*sdev;
}

double
nrnd2(const double& mean, const double& sdev)
{
    static int iset = 0;
    static double gset;
    double fac, r, v1, v2;

    if (iset == 0)
    {
        do
        {
            v1 = 2.0*rnd(0.0, 1.0) - 1.0;
            v2 = 2.0*rnd(0.0, 1.0) - 1.0;
            r = v1*v1 + v2*v2;
        }
        while (r >= 1.0);
        fac = sqrt(-2.0*log(r)/r);

        iset = 1;
        gset = v1*fac;
        return mean + v2*fac*sdev;
    }
    else
    {
        iset = 0;
        return mean + gset*sdev;
    }
}

double
nrnd3(const double& mean, const double& sdev, int next)
{
    return mean + nrndcache[next % NRNDCACHE_MAX]*sdev;
}

double
euclidean(vec &v1, vec &v2)
{
    vec dist = v1 - v2;
    dist *= dist;

    return dist.sum();
}

