/*
 *  $Id$
 *  Copyright (c) 2003, Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>
 *
 *  MoC - Master of Celebration is an automatic playlist generation tool using
 *  state-of-the-art music similarity measures
 *  http://www.schnitzer.at/dominik/moc/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __MOC_H
#define __MOC_H

#include <valarray>
#include <map>
#include <string>
#include "vec.h"
#include "mat.h"
#include "gmm.h"
#include "audiofilein.h"

using namespace std;

class moc
{
    private:
        mat *data;  // Matrix for the data
        mat mean;   // Gaussians x Dimension of a Gaussian - Mean Vectors
        mat cov;    // Gaussians x Dimension of a Gaussian - Covariance
        vec prior;  // Apriori probablilities per Gaussian (data/totaldata)
        string md5sum; // Md5sum of file beeing processed

        audiofilein ain;

        int gaussians;
        int gaussiandim;
        int window_size;
        int sample_rate;
        int mfcc_coeffs;
        int datacount;
        string db;

        double progr;
        int cancel;

        map<string, gmm*> mdb;

        int get_mfcc(const string &fname);
        void clusterby_em(int em);
        void clusterby_kmeans(int kmeans);
        void init_database();

    public:
        moc();
        // Initialize by given gata && evaluate means/covariances by kmeans
        moc(int gaussians, int window_size, int sample_rate, int mfcc_coeffs);
        // Initialize by given means/covariances && generate random featdata
        ~moc();

        gmm* get_gmm(const string &fname, int kmeans, int em);
        void add_gmm(gmm *g);
        string get_md5(const string &fname);
        
        gmm* get_gmm_md5(const string &md5);
        double get_progress();
        void get_nclosest_cancel();
        vector<pair<gmm*, double> >* get_nclosest(gmm *g, int count,
                int datacount);
};

#endif /* __MOC_H */
