/*
 *  $Id$
 *  Copyright (c) 2003, Dominik 'Aeneas' Schnitzer <dominik@schnitzer.at>
 *
 *  MoC - Master of Celebration is an automatic playlist generation tool using
 *  state-of-the-art music similarity measures
 *  http://www.schnitzer.at/dominik/moc/
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <iostream>
#include <valarray>

#include "fftw3.h"
#include "vec.h"
#include "stft.h"

using namespace std;

stft::stft(unsigned int winsize)
{
    this->winsize = winsize;
    fftsize = winsize*2;

    hamming = reinterpret_cast<double*>(malloc(sizeof(double) * winsize));
    for (unsigned int i = 0; i < winsize; i++)
        hamming[i] = 0.54 - 0.46 * cos(2*M_PIl * (i/(winsize-1)));

    data = reinterpret_cast<double*>(fftw_malloc(sizeof(double) * fftsize));
    plan = fftw_plan_r2r_1d(fftsize, data, data, FFTW_R2HC,
            FFTW_ESTIMATE | FFTW_DESTROY_INPUT);
}

stft::~stft()
{
    fftw_destroy_plan(plan);
    free(hamming);
    fftw_free(data);
}

void
stft::process(double *in, vec *out)
{
    // window the data using a hamming window
    memset(data, 0, sizeof(double) * fftsize);
    for (unsigned int i = 0; i < winsize; i++)
        data[i] = in[i] * hamming[i];

    fftw_execute(plan);

    out->resize(winsize);
    (*out)[0] = abs(data[0]);
    for (unsigned int i = 1; i < winsize/2; i++)
        (*out)[i] = sqrt(data[i]*data[i] + data[fftsize-i]*data[fftsize-i]);
    (*out)[winsize/2] = abs(data[winsize/2]);
    for (unsigned int i = winsize/2+1; i < winsize; i++)
        (*out)[i] = sqrt(data[i]*data[i] + data[fftsize-i]*data[fftsize-i]);
}

